#!/usr/bin/perl
#### Helper script to retrieve the download links for the Bundestag corpus.
#### This script is provided "as is" and any expressed or implied warranties, including, but not limited to, 
#### the implied warranties of merchantability and fitness for a particular purpose are disclaimed.
#### In no event shall the author be liable for any direct, indirect, incidental, special, exemplary, or 
#### consequential damages (including, but not limited to, procurement of substitute goods or services; loss of use, 
#### data, or profits; or business interruption) however caused and on any theory of liability, whether in contract,
#### strict liability, or tort (including negligence or otherwise) arising in any way out of the use of this software,
#### even if advised of the possibility of such damage.
#### Make sure you comply with the Bundestag's Terms of Use! At the time of this writing, the Conditions of Use could 
#### be found at http://www.bundestag.de/Mediathek/parlamentstv/nutzungsbedingungen.html

use strict;
use WWW::Mechanize;

## filenames of the corpus lists
my @permalinklists = ("train_list.txt", "devel_list.txt", "test_list.txt");
my $downloadlinkoutputfile = "downloadlinks.txt";

## correctly print umlauts
binmode(STDOUT, ":utf8");

## mechanised webbrowser
my $mech = WWW::Mechanize->new();

## load in all list files and concatenate them
my @allrecordings = ();
my $heading = "";
for my $list (@permalinklists) {
  open(my $fh, "<:encoding(UTF-8)", $list) or die "Can't open $list for reading.";
  my @curlist = <$fh>;
  close($fh);
  # remove line endings
  chomp(@curlist);  
  # remove heading of listfile
  $heading = shift(@curlist);
  @allrecordings = (@allrecordings, @curlist);
}

print("If you want to use the Bundestag speeches, you will have to check, accept, and make sure you comply with the Conditions of Use the Bundestag poses yourself. The author of this script takes no responsibility for your actions, has no influence on the contents of the Bundestag's website, and makes no guarantees about the data. Please check the source code of the script yourself to make sure it is fit for your system and won't cause any loss of data or other damage.\n");

## fetch terms of use url
my @firstline = split(/;/, $allrecordings[0]);
$mech->get($firstline[-1]);
my @termsofuselinks = $mech->find_link( text_regex => qr/Nutzungsbedingungen/i );
my $toulink = $termsofuselinks[0]->url_abs();

print("The Bundestag's Conditions of Use can be found here: $toulink\nIf this link is not valid, please search for the terms yourself, it is your responsibility to comply with them.\n");
print("Do you accept these terms? [NO/yes]: ");

my $useranswer = readline(*STDIN);
if(!($useranswer =~ m/yes/i)) {
  exit 0;
}

# open output file - it will have the original list info + the download link
open(my $out, ">:encoding(UTF-8)", $downloadlinkoutputfile) or die "Can't open $downloadlinkoutputfile for writing.";
print($out $heading . ";" . "downloadurl" . "\n");

## go over all recordings on all the lists
for my $recording (@allrecordings) {
  (my $lastname, my $firstname, my $date, my $time, my $url) = split(/;/, $recording);
  
  print("Retrieving info for speech $firstname $lastname, $date $time from $url ...\n");
  
  ## fetch the permalink webpage
  $mech->get($url);
  if($mech->status() != 200) {
    print("Can't retrieve permalink ".$url.", status: ".$mech->status()." \n");
    next;
  }
  
  ## search the webpage for links titled "Downloads"
  my @downloadlinks = $mech->find_link( text_regex => qr/Downloads/i );
  
  ## there should only be one download for the permalink. If not, you'll have to check yourself
  if(scalar(@downloadlinks) < 1) {
    print("Couldn't find download link for $url - please check manually.\n");
    next;
  }
  if(scalar(@downloadlinks) > 1) {
    print("More than 1 download link for $url - please check manually.\n");
    next;
  }

  ## the actual download url is hidden in the link's attribute data-downloadurl
  my %linkattributes = %{ $downloadlinks[0]->attrs() };
  my $downloadurl = $linkattributes{"data-downloadurl"};
  print("Download url: $downloadurl \n");
  
  ## write all info to output file
  print($out $recording . ";" . $downloadurl . "\n");
}

close($out);

print("\nDone. Check output file $downloadlinkoutputfile \n");
